<?php

namespace Drupal\purge_queuer_file_urls;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Url;
use Drupal\purge\Plugin\Purge\Invalidation\Exception\TypeUnsupportedException;
use Drupal\purge\Plugin\Purge\Invalidation\InvalidationsServiceInterface;
use Drupal\purge\Plugin\Purge\Queue\QueueServiceInterface;
use Drupal\purge\Plugin\Purge\Queuer\QueuerInterface;

/**
 * Base class for URL queuer classes.
 *
 * Implementations of this class will inject the dependencies required by the
 * specific queuer implementation. This includes the purge queuer plugin (to
 * determine if the queuer has been enabled by the user), any relevant settings,
 * and the invalidation type supported by the queuer.
 */
abstract class UrlQueuerBase implements UrlQueuerInterface {

  /**
   * The purge invalidation factory service.
   *
   * @var \Drupal\purge\Plugin\Purge\Invalidation\InvalidationsServiceInterface
   */
  protected $purgeInvalidationFactory;

  /**
   * The purge queue service.
   *
   * @var \Drupal\purge\Plugin\Purge\Queue\QueueServiceInterface
   */
  protected $purgeQueue;

  /**
   * The queuer plugin or NULL when disabled.
   *
   * @var \Drupal\purge\Plugin\Purge\Queuer\QueuerInterface
   */
  protected $purgeQueuerPlugin;

  /**
   * A list of URLs that have already been invalidated this request.
   *
   * Used to prevent the invalidation of the same URL multiple times.
   *
   * @var bool[]
   */
  protected $invalidatedUrls = [];

  /**
   * Constructs a new FileUrlQueuer.
   *
   * @param \Drupal\purge\Plugin\Purge\Invalidation\InvalidationsServiceInterface $purge_invalidation_factory
   *   The purge invalidation factory service.
   * @param \Drupal\purge\Plugin\Purge\Queue\QueueServiceInterface $purge_queue
   *   The purge queue service.
   * @param \Drupal\purge\Plugin\Queuer\Queuer $purge_queuer_plugin
   *   The purge queuer plugin.
   * @param \Drupal\purge_queuer_file_urls\UrlCollectorInterface $url_collector
   *   The URL collector.
   * @param string $invalidation_type
   *   The invalidation type.
   */
  public function __construct(InvalidationsServiceInterface $purge_invalidation_factory, QueueServiceInterface $purge_queue, QueuerInterface $purge_queuer_plugin) {
    $this->purgeInvalidationFactory = $purge_invalidation_factory;
    $this->purgeQueue = $purge_queue;
    $this->purgeQueuerPlugin = $purge_queuer_plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function invalidateUrls(iterable $urls) {
    if ($this->purgeQueuerPlugin) {
      $invalidations = [];
      /** @var \Drupal\purge_queuer_file_urls\UrlExpressionInterface $expression */
      foreach ($urls as $expression) {
        try {
          /** @var \Drupal\purge\Plugin\Purge\Invalidation\InvalidationInterface $invalidation */
          $invalidation = $expression->getInvalidation($this->purgeInvalidationFactory);
          $key = (string) $invalidation;
          if (empty($this->invalidatedUrls[$key])) {
            $this->invalidatedUrls[$key] = TRUE;
            $invalidations[] = $invalidation;
          }
        }
        catch (TypeUnsupportedException $e) {
          // A purger with URL support is not enabled.
          return;
        }
        catch (PluginNotFoundException $e) {
          // Uninstalling Purge or an otherwise stale plugin cache may prevent
          // the plugin from loading.
          return;
        }
      }
      if ($invalidations) {
        $this->purgeQueue->add($this->purgeQueuerPlugin, $invalidations);
      }
    }
  }

}
